<?php

function isAuthorized() {
  return isset($_SESSION["userid"]);
}

function isAdmin() {
  return isAuthorized() && $_SESSION["userrole"] == 0;
}

function insertDemand($row, $drow, $locations, $categories) {
  $row["owner"] = $_SESSION["userid"];
  insertRow("demands", $row);
  if (sqlOK()) {
    $demandid = getLastId();

    setDemandLocations($demandid, $locations);
    setDemandCategories($demandid, $categories);

    $drow["demand"] = $demandid;
    $src_lang = $drow["lang"];
    $drow["lang"] = sqlString($drow["lang"]);
    insertRow("demanddescriptions", $drow);

    if (insertDemandTranslation($demandid, $src_lang, "cs", $drow["name"], $drow["text"])) {
      if (sqlOK()) {
        moveAttachments($demandid);

        setPage("finished");
        $_SESSION["fintype"] = 2;
      } else
        sql("DELETE FROM demands WHERE id = ".$demandid);
    }
  }
}

function updateDemand($id, $row, $drow, $locations, $categories) {
  $row["id"] = $id;
  updateRow("demands", $row);
  if (sqlOK()) {
    setDemandLocations($id, $locations);
    setDemandCategories($id, $categories);

    sql("DELETE FROM demanddescriptions WHERE demand = ".$id);

    $drow["demand"] = $id;
    $src_lang = $drow["lang"];
    $drow["lang"] = sqlString($drow["lang"]);
    insertRow("demanddescriptions", $drow);

    if (insertDemandTranslation($id, $src_lang, "cs", $drow["name"], $drow["text"]))
      if (sqlOK()) {
        deleteAttachments($id);
        moveAttachments($id);

        setPage("finished");
        $_SESSION["fintype"] = 4;
      }
  }
}

function insertDemandTranslation($id, $src_lang, $dest_lang, $name, $text) {
  if ($src_lang == $dest_lang)
    return true;
  $gt = new GTranslate();
  $row["demand"] = $id;
  $row["lang"] = $dest_lang;
  try {
    $row["name"] = sqlString($gt->query(array(0 => $src_lang, 1 => $dest_lang), $name));
    $row["text"] = sqlString($gt->query(array(0 => $src_lang, 1 => $dest_lang), $text));
  } catch (GTranslateException $e) {
    setError("Translate error $src_lang => $dest_lang: ".$e->getMessage());
    return false;
  }
  insertRow("demanddescriptions", $row);
  return true;
}

function deleteAttachments($id) {
  for ($i = 1; $i <= 3; $i++)
    if (file_exists("attachments/$id/$i"))
      unlink("attachments/$id/$i");
  rmdir("attachments/$id");
}

function moveAttachments($id) {
  mkdir("attachments/$id");
  $tmptime = $_SESSION["tmptime"];
  for ($i = 1; $i <= 3; $i++)
    if (file_exists("tmp/$tmptime/$i"))
      rename("tmp/$tmptime/$i", "attachments/$id/$i");
  rmdir("tmp/$tmptime");
}

function setUserLocations($id, $locations) {
  updateJoinTable("userlocations", "user", "location", $id, $locations);
}

function setUserCategories($id, $categories) {
  updateJoinTable("usercategories", "user", "category", $id, $categories);
}

function setDemandLocations($id, $locations) {
  updateJoinTable("demandlocations", "demand", "location", $id, $locations);
}

function setDemandCategories($id, $categories) {
  updateJoinTable("demandcategories", "demand", "category", $id, $categories);
}

function searchTopCategoryIds($ids) {
  if ($ids && count($ids) > 0) {
    $ids_str = join(",", $ids);
    return sql("(SELECT c.id FROM categories c ".
      "WHERE c.parent = 0 AND c.id IN ($ids_str) ".
      "GROUP BY c.id)".
      "UNION (SELECT c.id FROM categories c ".
      "INNER JOIN categories c2 ON c.id = c2.parent ".
      "WHERE c.parent = 0 AND c2.id IN ($ids_str) ".
      "GROUP BY c.id)".
      "UNION (SELECT c.id FROM categories c ".
      "INNER JOIN categories c2 ON c.id = c2.parent ".
      "INNER JOIN categories c3 ON c2.id = c3.parent ".
      "WHERE c.parent = 0 AND c3.id IN ($ids_str) ".
      "GROUP BY c.id)");
  } else
    return array();
}

function searchUsersTopCategoryIds($userid) {
  $cats = sql("SELECT category FROM usercategories WHERE user = ".intval($userid));
  $ids = array();
  foreach ($cats as $row)
    add($ids, $row["category"]);
  return searchTopCategoryIds($ids);
}

function confirmDemand($id) {
  $codes = sql("SELECT l.code AS code FROM languages l WHERE NOT EXISTS (".
    "SELECT * FROM demanddescriptions dmd WHERE dmd.demand = $id AND dmd.lang = l.code)");
  $demand = sql("SELECT lang FROM demands WHERE id = $id LIMIT 0,1");
  $demanddesc = sql("SELECT * FROM demanddescriptions WHERE demand = $id AND lang = '".$demand[0]["lang"]."' LIMIT 0,1");
  foreach ($codes as $row)
    if (!insertDemandTranslation($id, $demand[0]["lang"], $row["code"], $demanddesc[0]["name"], $demanddesc[0]["text"])) {
      setError("Nepodařilo se přeložit poptávku na jazyk ".$row["code"]);
      return;
    }
  sql("UPDATE demands SET confirmed = true WHERE id = $id");
  $user_ids = sql("SELECT ids1.user AS user FROM ((SELECT DISTINCT uc.user AS user FROM usercategories uc, demandcategories dc ".
    "WHERE dc.category = uc.category AND dc.demand = $id) ".
    "UNION (SELECT DISTINCT uc.user AS user FROM usercategories uc, demandcategories dc ".
    "INNER JOIN categories c ON c.id = dc.category ".
    "WHERE dc.demand = $id AND c.parent = uc.category) ".
    "UNION (SELECT DISTINCT uc.user AS user FROM usercategories uc, demandcategories dc ".
    "INNER JOIN categories c ON c.id = dc.category ".
    "INNER JOIN categories c2 ON c2.id = c.parent ".
    "WHERE dc.demand = $id AND c2.parent = uc.category)) AS ids1, ".

    "((SELECT DISTINCT ul.user AS user FROM userlocations ul, demandlocations dl ".
    "WHERE dl.location = ul.location AND dl.demand = $id) ".
    "UNION (SELECT DISTINCT ul.user AS user FROM userlocations ul, demandlocations dl ".
    "INNER JOIN locations l ON l.id = dl.location ".
    "WHERE dl.demand = $id AND l.parent = ul.location) ".
    "UNION (SELECT DISTINCT ul.user AS user FROM userlocations ul, demandlocations dl ".
    "INNER JOIN locations l ON l.id = dl.location ".
    "INNER JOIN locations l2 ON l2.id = l.parent ".
    "WHERE dl.demand = $id AND l2.parent = ul.location)) AS ids2 WHERE ids1.user = ids2.user");
  foreach ($user_ids as $row) {
    echo $row["user"]."<br />";
    // odeslat email
  }
}

function checkValidDemands() {
  $expirated_demands = sql("SELECT dm.id, u.name, u.email FROM demands dm INNER JOIN users u ON u.id = dm.owner WHERE dm.valid = true AND dm.expiration < NOW()");
  foreach ($expirated_demands as $row) {
    // odeslat email, pokud uspech, tak odstranit validaci nasledujicim prikazem
    sql("UPDATE demands SET valid = false WHERE id = ".$row["id"]);
  }
}

function translateLocationsFromCS($max = 1000) {
  $to_translate = sql("SELECT l.id, lang.code, lcs.name FROM languages lang, locations l, locnames lcs ".
    "WHERE lang.code <> 'cs' AND NOT EXISTS (SELECT location FROM locnames ln ".
    "WHERE ln.location = l.id AND ln.lang = lang.code) AND ".
    "lcs.location = l.id AND lcs.lang = 'cs' LIMIT 0,$max");
  $gt = new GTranslate();
  $successes = 0;
  foreach ($to_translate as $row) {
    try {
      $translated = sqlString($gt->query(array(0 => "cs", 1 => $row["code"]), $row["name"]));
      sql("INSERT INTO locnames (location, lang, name) VALUES ($row[id], '$row[code]', $translated)");
      if (!sqlOK())
        echo "Cannot insert $row[name], id $row[id], to $row[code]<br />";
      else
        $successes++;
    } catch (GTranslateException $e) {
      echo "Can not translate $row[name], id $row[id], to $row[code]<br />";
    }
  }
  echo "Successes: $successes<br />";
}

?>
